@@ -94,10 +94,12 @@ e_phone_number_is_supported (void)
 
 /**
  * e_phone_number_get_country_code_for_region:
- * @region_code: (allow-none): a two-letter country code, or %NULL
+ * @region_code: (allow-none): a two-letter country code, a locale name, or
+ * %NULL
+ * @error: (out): a #GError to set an error, if any
  *
  * Retrieves the preferred country calling code for @region_code,
- * e.g. 358 for "fi".
+ * e.g. 358 for "fi" or 1 for "en_US@UTF-8".
  *
  * If %NULL is passed for @region_code the default region as returned by
  * e_phone_number_get_default_region() is used.
@@ -108,7 +110,8 @@ e_phone_number_is_supported (void)
  * Since: 3.8
  */
 gint
-e_phone_number_get_country_code_for_region (const gchar *region_code)
+e_phone_number_get_country_code_for_region (const gchar *region_code,
+                                            GError **error)
 {
 #ifdef ENABLE_PHONENUMBER
 
@@ -116,7 +119,7 @@ e_phone_number_get_country_code_for_region (const gchar *region_code)
 
 #else /* ENABLE_PHONENUMBER */
 
-	g_warning ("%s: The library was built without phone number support.", G_STRFUNC);
+	_e_phone_number_set_error (error, E_PHONE_NUMBER_ERROR_NOT_IMPLEMENTED);
 	return 0;
 
 #endif /* ENABLE_PHONENUMBER */
@@ -124,6 +127,7 @@ e_phone_number_get_country_code_for_region (const gchar *region_code)
 
 /**
  * e_phone_number_get_default_region:
+ * @error: (out): a #GError to set an error, if any
  *
  * Retrieves the current two-letter country code that's used by default for
  * parsing phone numbers in e_phone_number_from_string(). It can be useful
@@ -140,7 +144,7 @@ e_phone_number_get_country_code_for_region (const gchar *region_code)
  * Since: 3.8
  */
 gchar *
-e_phone_number_get_default_region (void)
+e_phone_number_get_default_region (GError **error)
 {
 #ifdef ENABLE_PHONENUMBER
 
@@ -148,7 +152,7 @@ e_phone_number_get_default_region (void)
 
 #else /* ENABLE_PHONENUMBER */
 
-	g_warning ("%s: The library was built without phone number support.", G_STRFUNC);
+	_e_phone_number_set_error (error, E_PHONE_NUMBER_ERROR_NOT_IMPLEMENTED);
 	return NULL;
 
 #endif /* ENABLE_PHONENUMBER */
@@ -215,6 +219,9 @@ e_phone_number_to_string (const EPhoneNumber *phone_number,
 
 #else /* ENABLE_PHONENUMBER */
 
+	/* The EPhoneNumber instance must be invalid. We'd also bail out with
+	 * a warning if phone numbers are supported. Any code triggering this
+	 * is broken and should be fixed. */
 	g_warning ("%s: The library was built without phone number support.", G_STRFUNC);
 	return NULL;
 
@@ -244,6 +251,9 @@ e_phone_number_get_country_code (const EPhoneNumber *phone_number,
 
 #else /* ENABLE_PHONENUMBER */
 
+	/* The EPhoneNumber instance must be invalid. We'd also bail out with
+	 * a warning if phone numbers are supported. Any code triggering this
+	 * is broken and should be fixed. */
 	g_warning ("%s: The library was built without phone number support.", G_STRFUNC);
 	return 0;
 
@@ -271,6 +281,9 @@ e_phone_number_get_national_number (const EPhoneNumber *phone_number)
 
 #else /* ENABLE_PHONENUMBER */
 
+	/* The EPhoneNumber instance must be invalid. We'd also bail out with
+	 * a warning if phone numbers are supported. Any code triggering this
+	 * is broken and should be fixed. */
 	g_warning ("%s: The library was built without phone number support.", G_STRFUNC);
 	return NULL;
 
@@ -298,9 +311,9 @@ e_phone_number_compare (const EPhoneNumber *first_number,
 
 #else /* ENABLE_PHONENUMBER */
 
-	/* NOTE: This calls for a dedicated return value, but I sense broken
-	 * client code that only checks for E_PHONE_NUMBER_MATCH_NONE and then
-	 * treats the "not-implemented" return value as a match */
+	/* The EPhoneNumber instance must be invalid. We'd also bail out with
+	 * a warning if phone numbers are supported. Any code triggering this
+	 * is broken and should be fixed. */
 	g_warning ("%s: The library was built without phone number support.", G_STRFUNC);
 	return E_PHONE_NUMBER_MATCH_NONE;
 
@@ -323,10 +336,34 @@ EPhoneNumberMatch
 e_phone_number_compare_strings (const gchar *first_number,
                                 const gchar *second_number,
                                 GError **error)
+{
+	return e_phone_number_compare_strings_with_region (
+		first_number, second_number, NULL, error);
+}
+
+/**
+ * e_phone_number_compare_strings_with_region:
+ * @first_number: the first EPhoneNumber to compare
+ * @second_number: the second EPhoneNumber to compare
+ * @region_code: (allow-none): a two-letter country code, or %NULL
+ * @error: (out): a #GError to set an error, if any
+ *
+ * Compares two phone numbers within the context of @region_code.
+ *
+ * Returns: The quality of matching for the two phone numbers.
+ *
+ * Since: 3.8
+ **/
+EPhoneNumberMatch
+e_phone_number_compare_strings_with_region (const gchar *first_number,
+                                            const gchar *second_number,
+                                            const gchar *region_code,
+                                            GError **error)
 {
 #ifdef ENABLE_PHONENUMBER
 
-	return _e_phone_number_cxx_compare_strings (first_number, second_number, error);
+	return _e_phone_number_cxx_compare_strings (
+		first_number, second_number, region_code, error);
 
 #else /* ENABLE_PHONENUMBER */
 
