@@ -581,6 +581,9 @@ gboolean
 camel_imapx_command_set_error_if_failed (CamelIMAPXCommand *ic,
                                          GError **error)
 {
+	CamelIMAPXJob *job;
+	GCancellable *cancellable = NULL;
+
 	g_return_val_if_fail (CAMEL_IS_IMAPX_COMMAND (ic), FALSE);
 
 	if (ic->status != NULL && ic->status->result != IMAPX_OK) {
@@ -607,7 +610,11 @@ camel_imapx_command_set_error_if_failed (CamelIMAPXCommand *ic,
 		return TRUE;
 	}
 
-	return FALSE;
+	job = camel_imapx_command_get_job (ic);
+	if (job != NULL)
+		cancellable = camel_imapx_job_get_cancellable (job);
+
+	return g_cancellable_set_error_if_cancelled (cancellable, error);
 }
 
 CamelIMAPXCommandQueue *
