@@ -188,18 +188,10 @@ fill_fi (CamelStore *store,
 	local_store = CAMEL_LOCAL_STORE (store);
 	folder = camel_object_bag_peek (store->folders, fi->full_name);
 
-	if (folder == NULL
-	    && (flags & CAMEL_STORE_FOLDER_INFO_FAST) == 0)
-		folder = camel_store_get_folder_sync (
-			store, fi->full_name, 0, cancellable, NULL);
-
 	if (folder != NULL) {
-		if ((flags & CAMEL_STORE_FOLDER_INFO_FAST) == 0)
-			camel_folder_refresh_info_sync (folder, cancellable, NULL);
 		fi->unread = camel_folder_get_unread_message_count (folder);
 		fi->total = camel_folder_get_message_count (folder);
 		g_object_unref (folder);
-
 	} else {
 		CamelLocalSettings *local_settings;
 		CamelSettings *settings;
