@@ -396,8 +396,6 @@ connect_to_server (CamelService *service,
 		goto fail;
 	}
 
-	nntp_store_reset_state (nntp_store, nntp_stream);
-
 	/* backward compatibility, empty 'mechanism' is a non-migrated account */
 	if ((user != NULL && *user != '\0' && (!mechanism || !*mechanism)) ||
 	    (mechanism && *mechanism && g_strcmp0 (mechanism, "ANONYMOUS") != 0)) {
@@ -416,6 +414,8 @@ connect_to_server (CamelService *service,
 			goto fail;
 	}
 
+	nntp_store_reset_state (nntp_store, nntp_stream);
+
 	/* set 'reader' mode & ignore return code, also ping the server, inn goes offline very quickly otherwise */
 	if (camel_nntp_raw_command_auth (nntp_store, cancellable, error, (gchar **) &buf, "mode reader") == -1
 	    || camel_nntp_raw_command_auth (nntp_store, cancellable, error, (gchar **) &buf, "date") == -1)
@@ -1161,7 +1161,7 @@ nntp_store_get_folder_info_all (CamelNNTPStore *nntp_store,
                                 GCancellable *cancellable,
                                 GError **error)
 {
-	CamelNNTPStream *nntp_stream = NULL;
+	CamelNNTPStream *nntp_stream;
 	CamelNNTPStoreSummary *nntp_store_summary;
 	CamelNNTPStoreInfo *si;
 	guint len;
@@ -1169,6 +1169,7 @@ nntp_store_get_folder_info_all (CamelNNTPStore *nntp_store,
 	gint ret = -1;
 	CamelFolderInfo *fi = NULL;
 
+	nntp_stream = camel_nntp_store_ref_stream (nntp_store);
 	nntp_store_summary = camel_nntp_store_ref_summary (nntp_store);
 
 	if (top == NULL)
@@ -1196,8 +1197,6 @@ nntp_store_get_folder_info_all (CamelNNTPStore *nntp_store,
 				goto do_complete_list;
 			}
 
-			nntp_stream = camel_nntp_store_ref_stream (nntp_store);
-
 			while ((ret = camel_nntp_stream_line (nntp_stream, &line, &len, cancellable, error)) > 0)
 				nntp_store_info_update (nntp_store, (gchar *) line);
 		} else {
@@ -1224,8 +1223,6 @@ nntp_store_get_folder_info_all (CamelNNTPStore *nntp_store,
 			for (i = 0; (si = (CamelNNTPStoreInfo *) camel_store_summary_index (CAMEL_STORE_SUMMARY (nntp_store_summary), i)); i++)
 				g_hash_table_insert (all, si->info.path, si);
 
-			nntp_stream = camel_nntp_store_ref_stream (nntp_store);
-
 			while ((ret = camel_nntp_stream_line (nntp_stream, &line, &len, cancellable, error)) > 0) {
 				si = nntp_store_info_update (nntp_store, (gchar *) line);
 				g_hash_table_remove (all, si->info.path);
@@ -1292,17 +1289,8 @@ nntp_get_folder_info_online (CamelStore *store,
                              GCancellable *cancellable,
                              GError **error)
 {
-	CamelService *service;
-	CamelFolderInfo *info = NULL;
-
-	service = CAMEL_SERVICE (store);
-
-	/* Reconnect if necessary. */
-	if (camel_service_connect_sync (service, cancellable, error))
-		info = nntp_get_folder_info (
-			store, top, flags, TRUE, cancellable, error);
-
-	return info;
+	return nntp_get_folder_info (
+		store, top, flags, TRUE, cancellable, error);
 }
 
 static CamelFolderInfo *
@@ -2287,15 +2275,13 @@ camel_nntp_command (CamelNNTPStore *nntp_store,
 		case 400:	/* service discontinued */
 		case 401:	/* wrong client state - this should quit but this is what the old code did */
 		case 503:	/* information not available - this should quit but this is what the old code did (?) */
-			if (camel_service_get_connection_status (service) != CAMEL_SERVICE_CONNECTING)
-				camel_service_disconnect_sync (
-					service, FALSE, cancellable, NULL);
+			camel_service_disconnect_sync (
+				service, FALSE, cancellable, NULL);
 			ret = -1;
 			continue;
 		case -1:	/* i/o error */
-			if (camel_service_get_connection_status (service) != CAMEL_SERVICE_CONNECTING)
-				camel_service_disconnect_sync (
-					service, FALSE, cancellable, NULL);
+			camel_service_disconnect_sync (
+				service, FALSE, cancellable, NULL);
 			if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED) || retry >= 3) {
 				g_propagate_error (error, local_error);
 				ret = -1;
