@@ -47,7 +47,9 @@
 #undef MIN
 #undef MAX
 
-#define d(x) (camel_debug ("smtp") ? (x) : 0)
+extern gint camel_verbose_debug;
+
+#define d(x) (camel_verbose_debug ? (x) : 0)
 
 /* Specified in RFC 821 */
 #define SMTP_PORT  25
@@ -184,6 +186,9 @@ connect_to_server (CamelService *service,
 		transport->flags &= ~CAMEL_SMTP_TRANSPORT_IS_ESMTP;
 
 		if (!smtp_helo (transport, cancellable, error)) {
+			camel_service_disconnect_sync (
+				CAMEL_SERVICE (transport),
+				TRUE, cancellable, NULL);
 			success = FALSE;
 			goto exit;
 		}
@@ -247,6 +252,9 @@ connect_to_server (CamelService *service,
 	/* We are supposed to re-EHLO after a successful STARTTLS to
 	 * re-fetch any supported extensions. */
 	if (!smtp_helo (transport, cancellable, error)) {
+		camel_service_disconnect_sync (
+			CAMEL_SERVICE (transport),
+			TRUE, cancellable, NULL);
 		success = FALSE;
 	}
 
@@ -260,14 +268,13 @@ exception_cleanup:
 	g_object_unref (transport->ostream);
 	transport->ostream = NULL;
 
+	transport->connected = FALSE;
+
 	success = FALSE;
 
 exit:
 	g_free (host);
 
-	if (!success)
-		transport->connected = FALSE;
-
 	return success;
 }
 
@@ -424,7 +431,8 @@ smtp_transport_connect_sync (CamelService *service,
 		g_object_unref (session);
 
 		if (!success)
-			transport->connected = FALSE;
+			camel_service_disconnect_sync (
+				service, TRUE, cancellable, NULL);
 	}
 
 exit:
@@ -1197,6 +1205,10 @@ smtp_helo (CamelSmtpTransport *transport,
 		g_prefix_error (error, _("HELO command failed: "));
 		camel_operation_pop_message (cancellable);
 
+		camel_service_disconnect_sync (
+			CAMEL_SERVICE (transport),
+			FALSE, cancellable, NULL);
+
 		return FALSE;
 	}
 	g_free (cmdbuf);
