@@ -77,6 +77,8 @@ void		e_util_free_nullable_object_slist
 						(GSList *objects);
 void		e_queue_transfer		(GQueue *src_queue,
 						 GQueue *dst_queue);
+GWeakRef *	e_weak_ref_new			(gpointer object);
+void		e_weak_ref_free			(GWeakRef *weak_ref);
 
 gboolean	e_file_recursive_delete_sync	(GFile *file,
 						 GCancellable *cancellable,
@@ -126,11 +128,6 @@ gchar *		e_util_replace_prefix		(const gchar *configure_time_prefix,
 						 const gchar *configure_time_path);
 #endif
 
-gint		e_data_server_util_get_dbus_call_timeout
-						(void);
-void		e_data_server_util_set_dbus_call_timeout
-						(gint timeout_msec);
-
 /* utility functions for easier processing of named parameters */
 
 /**
@@ -168,6 +165,11 @@ GSList *	e_util_copy_string_slist	(GSList *copy_to,
 						 const GSList *strings);
 GSList *	e_util_copy_object_slist	(GSList *copy_to,
 						 const GSList *objects);
+gint		e_data_server_util_get_dbus_call_timeout
+						(void);
+void		e_data_server_util_set_dbus_call_timeout
+						(gint timeout_msec);
+
 #endif /* EDS_DISABLE_DEPRECATED */
 
 G_END_DECLS
