@@ -162,6 +162,11 @@ static guint signals[LAST_SIGNAL];
 /* Forward Declarations */
 static void	e_source_initable_init		(GInitableIface *interface);
 
+/* Private function shared only with ESourceRegistry. */
+void		__e_source_private_replace_dbus_object
+						(ESource *source,
+						 GDBusObject *dbus_object);
+
 G_DEFINE_TYPE_WITH_CODE (
 	ESource,
 	e_source,
@@ -1730,6 +1735,28 @@ e_source_init (ESource *source)
 	g_rec_mutex_init (&source->priv->lock);
 }
 
+void
+__e_source_private_replace_dbus_object (ESource *source,
+                                        GDBusObject *dbus_object)
+{
+	/* XXX This function is only ever called by ESourceRegistry
+	 *     when recovering from a registry service restart.  It
+	 *     replaces the defunct proxy object with an equivalent
+	 *     proxy object for the newly-started registry service. */
+
+	g_return_if_fail (E_IS_SOURCE (source));
+	g_return_if_fail (E_DBUS_IS_OBJECT (dbus_object));
+
+	g_mutex_lock (&source->priv->property_lock);
+
+	g_clear_object (&source->priv->dbus_object);
+	source->priv->dbus_object = g_object_ref (dbus_object);
+
+	g_mutex_unlock (&source->priv->property_lock);
+
+	g_object_notify (G_OBJECT (source), "dbus-object");
+}
+
 /**
  * e_source_new:
  * @dbus_object: (allow-none): a #GDBusObject or %NULL
@@ -2323,12 +2350,18 @@ e_source_has_extension (ESource *source,
 GDBusObject *
 e_source_ref_dbus_object (ESource *source)
 {
+	GDBusObject *dbus_object = NULL;
+
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
 
-	if (source->priv->dbus_object == NULL)
-		return NULL;
+	g_mutex_lock (&source->priv->property_lock);
 
-	return g_object_ref (source->priv->dbus_object);
+	if (source->priv->dbus_object != NULL)
+		dbus_object = g_object_ref (source->priv->dbus_object);
+
+	g_mutex_unlock (&source->priv->property_lock);
+
+	return dbus_object;
 }
 
 /**
@@ -2794,7 +2827,7 @@ e_source_remote_create_sync (ESource *source,
  * possibly a server-side path or ID for the resource.
  *
  * When the operation is finished, @callback will be called.  You can then
- * call 3_source_remote_create_finish() to get the result of the operation.
+ * call e_source_remote_create_finish() to get the result of the operation.
  *
  * Since: 3.6
  **/
@@ -2951,8 +2984,7 @@ e_source_remote_delete_finish (ESource *source,
  * @error: return location for a #GError, or %NULL
  *
  * Obtains the OAuth 2.0 access token for @source along with its expiry
- * in seconds from the current time (or 0 if unknown).  The @source must
- * have #ESource:supports-oauth2 set for this to work.
+ * in seconds from the current time (or 0 if unknown).
  *
  * Free the returned access token with g_free() when finished with it.
  * If an error occurs, the function will set @error and return %FALSE.
@@ -2990,7 +3022,6 @@ e_source_get_oauth2_access_token_sync (ESource *source,
  *
  * Asynchronously obtains the OAuth 2.0 access token for @source along
  * with its expiry in seconds from the current time (or 0 if unknown).
- * The @source must have #ESource:supports-oauth2 set for this to work.
  *
  * When the operation is finished, @callback will be called.  You can then
  * call e_source_get_oauth2_access_token_finish() to get the result of the
