@@ -72,19 +72,36 @@ main (gint argc,
 	g_type_init ();
 #endif
 	g_test_init (&argc, &argv, NULL);
+	g_test_bug_base ("http://bugzilla.gnome.org/");
 
 	g_test_add (
-		"/EBookClient/GetContact/Sync", ETestServerFixture, &book_closure,
-		e_test_server_utils_setup, test_get_contact_sync, e_test_server_utils_teardown);
+		"/EBookClient/GetContact/Sync",
+		ETestServerFixture,
+		&book_closure,
+		e_test_server_utils_setup,
+		test_get_contact_sync,
+		e_test_server_utils_teardown);
 	g_test_add (
-		"/EBookClient/GetContact/Async", ETestServerFixture, &book_closure,
-		e_test_server_utils_setup, test_get_contact_async, e_test_server_utils_teardown);
+		"/EBookClient/GetContact/Async",
+		ETestServerFixture,
+		&book_closure,
+		e_test_server_utils_setup,
+		test_get_contact_async,
+		e_test_server_utils_teardown);
 	g_test_add (
-		"/EBookClient/DirectAccess/GetContact/Sync", ETestServerFixture, &direct_book_closure,
-		e_test_server_utils_setup, test_get_contact_sync, e_test_server_utils_teardown);
+		"/EBookClient/DirectAccess/GetContact/Sync",
+		ETestServerFixture,
+		&direct_book_closure,
+		e_test_server_utils_setup,
+		test_get_contact_sync,
+		e_test_server_utils_teardown);
 	g_test_add (
-		"/EBookClient/DirectAccess/GetContact/Async", ETestServerFixture, &direct_book_closure,
-		e_test_server_utils_setup, test_get_contact_async, e_test_server_utils_teardown);
+		"/EBookClient/DirectAccess/GetContact/Async",
+		ETestServerFixture,
+		&direct_book_closure,
+		e_test_server_utils_setup,
+		test_get_contact_async,
+		e_test_server_utils_teardown);
 
 	return e_test_server_utils_run ();
 }
