@@ -211,13 +211,22 @@ main (gint argc,
 	g_type_init ();
 #endif
 	g_test_init (&argc, &argv, NULL);
+	g_test_bug_base ("http://bugzilla.gnome.org/");
 
 	g_test_add (
-		"/ECalClient/GetView/Sync", ETestServerFixture, &cal_closure,
-		e_test_server_utils_setup, test_get_view_sync, e_test_server_utils_teardown);
+		"/ECalClient/GetView/Sync",
+		ETestServerFixture,
+		&cal_closure,
+		e_test_server_utils_setup,
+		test_get_view_sync,
+		e_test_server_utils_teardown);
 	g_test_add (
-		"/ECalClient/GetView/Async", ETestServerFixture, &cal_closure,
-		e_test_server_utils_setup, test_get_view_async, e_test_server_utils_teardown);
+		"/ECalClient/GetView/Async",
+		ETestServerFixture,
+		&cal_closure,
+		e_test_server_utils_setup,
+		test_get_view_async,
+		e_test_server_utils_teardown);
 
 	return e_test_server_utils_run ();
 }
